@@ -3,8 +3,8 @@
   Program:   Insight Segmentation & Registration Toolkit
   Module:    $RCSfile: itkQuadEdgeMesh.h,v $
   Language:  C++
-  Date:      $Date: 2009-02-07 23:16:28 $
-  Version:   $Revision: 1.33 $
+  Date:      $Date: 2009-04-08 23:10:20 $
+  Version:   $Revision: 1.34 $
 
   Copyright (c) Insight Software Consortium. All rights reserved.
   See ITKCopyright.txt or http://www.itk.org/HTML/Copyright.htm for details.
@@ -483,6 +483,9 @@ protected:
   QuadEdgeMesh();
   virtual ~QuadEdgeMesh(); 
 
+  /** Release the memory of each one of the cells independently. */
+  virtual void ClearCellsContainer();
+
   CellsContainerPointer  m_EdgeCellsContainer;
 
 private:
